@@ -46,6 +46,8 @@
 #include <stdio.h>
 #include <string.h>
 #include <fcntl.h>
+#include <limits.h>
+
 #ifdef GETOPTLONG
 #include <getopt.h>
 #endif /* GETOPTLONG */
@@ -126,7 +128,7 @@ static void usage()
 "  -s\t-- eject SCSI device\n"
 "  -f\t-- eject floppy\n"
 "  -q\t-- eject tape\n"
-"  -p\t-- use /proc/mounts instead /etc/mtab\n")
+"  -p\t-- use /proc/mounts instead of /etc/mtab\n")
 , version);
 #ifdef GETOPTLONG
 	fprintf(stderr,_(
@@ -273,14 +275,16 @@ static void parse_args(int argc, char **argv, char **device)
 /* Return 1 if file/device exists, 0 otherwise. */
 static int FileExists(const char *name)
 {
-	int fd;
 
-	fd = open(name, O_RDONLY|O_NONBLOCK);
-	if (fd == -1) {
-		return 0;
-	} else {
-		close(fd);
+	/*
+	 * access() uses the UID, not the EUID. This way a normal user
+	 * cannot find out if a file (say, /root/fubar) exists or not, even
+	 * if eject is SUID root
+	 */
+	if (access (name, F_OK) == 0) {
 		return 1;
+	}	else {
+		return 0;
 	}
 }
 
@@ -712,12 +716,18 @@ static void UnmountDevices(const char *pattern)
 static char *SymLink(const char *name)
 {
 	int status;
-	char s1[1024];
-	char s2[1024];
-	char s4[1024];
+	char s1[PATH_MAX];
+	char s2[PATH_MAX];
+	char s4[PATH_MAX];
+	char result[PATH_MAX];
 	char *s3;
 
-	status = readlink(name, s1, sizeof(s1));
+	memset(s1, 0, sizeof(s1));
+	memset(s2, 0, sizeof(s2));
+	memset(s4, 0, sizeof(s4));
+	memset(result, 0, sizeof(result));
+
+	status = readlink(name, s1, sizeof(s1) - 1);
 
 	if (status == -1)
 		return 0;
@@ -730,10 +740,10 @@ static char *SymLink(const char *name)
 		s3 = strrchr(s2, '/');
 		if (s3 != 0) {
 			s3[1] = 0;
-			strcat(s2, s1);
+			snprintf(result, sizeof(result)-1, "%s%s", s2, s1);
 		}
 	}
-	realpath(s2, s4);
+	realpath(result, s4);
 	return strdup(s4);
 }
 
